<?php
/*
Plugin Name: Comvatars
Plugin URI: http://soeren-weber.net/post/2005/08/07/44
Description: Shows avatar images on user comments to uniquely identify the author much like various bulletin board systems do. At least the <a href="http://www.skippy.net/blog/2005/03/24/gravatars/">Gravatars plugin by Skippy</a> or the <a href="http://dev.wp-plugins.org/wiki/favatars">Favatars plugin by Jeff Minard</a> must be installed and activated. Please visit the plugin homepage for further documentation.
Version: 1.5
Author: S&ouml;ren Weber
Author URI: http://soeren-weber.net/
*/

/* set this value to zero if you don't want the plugin to use the appropirate
hooks; this may be useful if you prefer to use the template tags */
define(COMVATAR_USE_HOOKS, 1);

/* the current version number of this file; MUST be identical to the above
documented 'Version:' comment */
define(COMVATAR_VERSION, '1.5');

/* the path relative to your WP installation where to look for additional
Comvatar plugin files */
define(COMVATAR_BASE_PATH, '/wp-content/comvatars/');

/* the path relative to COMVATAR_BASE_PATH where to look for the default
collection, the path MUST end with a '/' otherwise */
define(COMVATAR_COLLECTION_PATH, 'collection/');

/* this image will used in case no user_name, user_email or default image
is chosen; it must reside in COMVATAR_BASE_PATH */
define(COMVATAR_ANONYMOUS_AVATAR, 'anonymous.png');

/* we can not take this 'Anonymous' string from the localization file,
because it's hardcoded (at least WP 1.5.1.3 and below) */
define(COMVATAR_ANONYMOUS_NAME, 'Anonymous');

/* in case we haven't an email address we set a default to force
gravatars.com to execute our request; this will not be displayed */
define(COMVATAR_ANONYMOUS_EMAIL, 'force_gravatar@example.com');

/* favicons are almost every time only 16x16 px, so we use this as default */
define(COMVATAR_DEFAULT_ICON_SIZE, 16);

/* this activates the snoopy class, that tries to find out whether an
ULR is redirected and uses this redirected URL for the favatar retrieval;
in case you are extensively uses <!--comvatar --> tags with http addresses
in your posts this feature can slow down the loading of your site
dramatically */
define(COMVATAR_FAVATAR_REDIRECTION, 1);

// --------------------------------------------------------------------------
// The Comvatar class, the runtime master brain of this plugin
// --------------------------------------------------------------------------

class Comvatar
{
  var $version = COMVATAR_VERSION;
  var $m_base_path = COMVATAR_BASE_PATH;
  var $m_collection_path = '';
  var $m_collection_files = array();
  var $m_anonymous_avatar = COMVATAR_ANONYMOUS_AVATAR;
  var $m_gravatar_plugin_installed = false;
  var $m_favatar_plugin_installed = false;
  var $m_custom_size = COMVATAR_DEFAULT_ICON_SIZE;
  var $m_snoopy;

  function Comvatar()
  {
    $this->m_collection_path = $this->m_base_path. COMVATAR_COLLECTION_PATH;

    // find out whether depending base plugins are installed and activated;
    // code borrowed from Plugins Used; you'll get it back after we finished here ;)
    $activePluginsResult = get_settings('active_plugins');
    if (is_array($activePluginsResult)) {
      $pup_plugin_files = $activePluginsResult;
    } else {
      $pup_plugin_files = explode("\n",$activePluginsResult);
    }
    $myPluginFiles = array_values($pup_plugin_files);
    if (is_array($myPluginFiles[0])) {
      $myPluginFiles = array_keys($pup_plugin_files);
    }
    $myPluginFiles=array_unique($myPluginFiles);
    foreach($myPluginFiles as $plugin_file) {
      if (trim($plugin_file) == "")
        continue;
      else if (trim($plugin_file) == "gravatars.php") {
        global $grav_options;
        if (!isset($grav_options))
          continue;
        $this->m_gravatar_plugin_installed = true;
        continue;
      } else if (trim($plugin_file) == "favatars.php") {
        $this->m_favatar_plugin_installed = true;
        continue;
      }
    }

    // fix gravatar options
    if ($this->m_gravatar_plugin_installed)
    {
      global $grav_options;
      $this->m_custom_size = $grav_options['gravatar_size'];
      $this->m_collection_path = $grav_options['gravatar_default'];
      gravatar_defaults('update', $grav_options['gravatar_rating'],
        $this->m_custom_size, $grav_options['gravatar_border'],
        $this->m_collection_path, 0, $grav_options['gravatar_in_posts'], 0, 0);
    }

    // read collection
    $collection_dir_path = ABSPATH. $this->m_collection_path;
    $collection_dir = @dir($collection_dir_path);
    if ($collection_dir) {
      while(($file_name = $collection_dir->read()) !== false)
      {
        if (is_file($collection_dir_path.'/'.$file_name) && $file_name !=  ".." && $file_name !=  ".")
          $this->m_collection_files[] = $file_name;
      }
    }
    sort($this->m_collection_files);

    // this installs redirection support for favatar detection
    if ($this->m_favatar_plugin_installed && COMVATAR_FAVATAR_REDIRECTION)
    {
      require_once(ABSPATH. 'wp-includes/class-snoopy.php');
      $this->m_snoopy = new Snoopy();
      remove_action('pingback_post', 'discover_favicon');
      remove_action('trackback_post', 'discover_favicon');
      remove_action('comment_post', 'discover_favicon');
      add_action('comment_post', array($this, 'comvatar_comment_discover_favicon'));
      add_action('trackback_post', array($this, 'comvatar_comment_discover_favicon'));
      add_action('pingback_post', array($this, 'comvatar_comment_discover_favicon'));
    }

    // install callbacks
    if (COMVATAR_USE_HOOKS)
    {
      add_filter('get_comment_author_link', array($this, 'comvatar_comment_image_custom'));
      add_filter('comment_author', array($this, 'comvatar_comment_image_default'));
    }
    add_filter('the_content', array($this, 'comvatar_xhtml_tag'));
  }

  function select_comvatar_from_collection($user_name, $user_email, $default_avatar_url='')
  {
    if (empty($this->m_collection_files))
      return @trim($default_avatar_url);
    $hash = crc32($user_name.'|'.$user_email);
    $index = abs($hash % count($this->m_collection_files));
    return get_settings('siteurl'). $this->m_collection_path . $this->m_collection_files[$index];
  }

  function get_comment_author_url()
  {
    global $wpdb, $comment;

    if (isset($comment) && isset($comment->comment_ID) && is_numeric($comment->comment_ID))
      $url = @trim(@$wpdb->get_var(
        "SELECT `comment_author_url` FROM `$wpdb->comments` WHERE `comment_ID` = '$comment->comment_ID'"));
    if (!isset($url))
      $url = '';
    return $url;
  }

  function get_comment_author_email()
  {
    global $wpdb, $comment;

    if (isset($comment) && isset($comment->comment_ID) && is_numeric($comment->comment_ID))
      $email = @trim(@$wpdb->get_var(
        "SELECT `comment_author_email` FROM `$wpdb->comments` WHERE `comment_ID` = '$comment->comment_ID'"));
    if (!isset($email))
      $email = '';
    return $email;
  }

  function get_comment_author_name()
  {
    global $wpdb, $comment;

    if (isset($comment) && isset($comment->comment_ID) && is_numeric($comment->comment_ID))
      $name = @trim(@$wpdb->get_var(
        "SELECT `comment_author` FROM `$wpdb->comments` WHERE `comment_ID` = '$comment->comment_ID'"));
    if (!isset($name))
      $name = '';
    return $name;
  }

  function get_string_type($string='', &$name, &$email, &$url)
  {
    $name = '';
    $email = '';
    $url = '';
    $string = @trim($string);
    if (is_email($string))
    {
      $email = $string;
      return 1;
    }
    else if (($r = strpos(strtolower(trim($string)), "http://")) !== false && $r == 0)
    {
      $url = $string;
      return 2;
    }
    $name = $string;
    return 0;
  }

  function get_comvatar_url(&$user_name, $user_email='', $default_avatar_url='')
  {
    $user_name = @trim($user_name);
    if (empty($user_name))
      $user_name = COMVATAR_ANONYMOUS_NAME;

    $user_email = @trim($user_email);
    if (empty($user_email))
      $user_email = COMVATAR_ANONYMOUS_EMAIL;

    if ($user_name == COMVATAR_ANONYMOUS_NAME && $user_email == COMVATAR_ANONYMOUS_EMAIL)
      $avatar_url = @trim($default_avatar_url);
    else if (($r = strpos(strtolower(@trim($this->m_collection_path)), "http://")) !== false && $r == 0)
      $avatar_url = @trim($this->m_collection_path);
    else if (is_file(ABSPATH. $this->m_collection_path))
      $avatar_url = get_settings('siteurl'). $this->m_collection_path;
    else if (is_dir(ABSPATH. $this->m_collection_path))
      $avatar_url = $this->select_comvatar_from_collection($user_name, $user_email, $default_avatar_url);
    else
      $avatar_url = @trim($default_avatar_url);
    return $avatar_url;
  }

  function get_favatar_url($user_url, $default_avatar_url='')
  {
    global $wpdb, $comment;

    if ($this->m_favatar_plugin_installed)
    {
      if (isset($comment) && isset($comment->comment_ID) && is_numeric($comment->comment_ID))
        $avatar_url = @trim(@$wpdb->get_var(
          "SELECT `comment_favicon_url` FROM `$wpdb->comments` WHERE `comment_ID` = '$comment->comment_ID'"));
      else if (isset($user_url) && !empty($user_url))
      {
        if (COMVATAR_FAVATAR_REDIRECTION && $this->m_snoopy->fetch($user_url))
          $user_url = $this->m_snoopy->lastredirectaddr;
        $avatar_url = getFavicon($user_url);
      }
    }
    if (!isset($avatar_url) || empty($avatar_url))
      $avatar_url = @trim($default_avatar_url);
    return $avatar_url;
  }

  function get_gravatar_url($user_email='', &$icon_size, $default_avatar_url='')
  {
    $user_email = @trim($user_email);
    if (empty($user_email))
      $user_email = COMVATAR_ANONYMOUS_EMAIL;

    $icon_size = trim($icon_size);
    if (empty($icon_size) || !is_numeric($icon_size))
      $icon_size = $this->m_custom_size;

    if ($this->m_gravatar_plugin_installed)
    {
      global $grav_options;
      $saved_gravatar_size = $grav_options['gravatar_size'];
      $grav_options['gravatar_size'] = $icon_size;
      $avatar_url = wp_gravatar($user_email, $default_avatar_url);
      $grav_options['gravatar_size'] = $saved_gravatar_size;
    }
    // in case gravatar.com is unavailable
    if (!isset($avatar_url) || !$avatar_url || empty($avatar_url))
      $avatar_url = @trim($default_avatar_url);
    return $avatar_url;
  }

  function get_smart_avatar_url(&$user_name, &$user_email,
    &$user_url, &$icon_size, $default_avatar_url='')
  {
    if (empty($user_name))
      $user_name = $this->get_comment_author_name();
    if (empty($user_email))
      $user_email = $this->get_comment_author_email();
    if (empty($user_url))
      $user_url = $this->get_comment_author_url();

    if (empty($default_avatar_url))
      $default_avatar_url = get_settings('siteurl').
        $this->m_base_path. $this->m_anonymous_avatar;
    $default_avatar_url = $this->get_comvatar_url($user_name, $user_email, $default_avatar_url);
    $default_avatar_url = $this->get_favatar_url($user_url, $default_avatar_url);
    return $this->get_gravatar_url($user_email, $icon_size, $default_avatar_url);
  }

  function get_smart_avatar_image(&$user_name, &$user_email,
    &$user_url, &$icon_size, $default_avatar_url='',
    $before='<img class="comvatar"', $after=' />')
  {
    $avatar_url = $this->get_smart_avatar_url($user_name, $user_email,
      $user_url, $icon_size, $default_avatar_url);
    $html = $before. ' src="'. $avatar_url.
      '" height="'. $icon_size. '" width="'. $icon_size.
      '" alt="'. $user_name. '"'. $after;
    return $html;
  }

  function get_smart_avatar_image_link(&$user_name, &$user_email,
    &$user_url, &$icon_size, $default_avatar_url='')
  {
    $avatar_image = $this->get_smart_avatar_image($user_name, $user_email,
      $user_url, $icon_size, $default_avatar_url);
    $html = '';
    if (!empty($user_url))
      $html .= '<a href="'. $user_url. '">';
    $html .= $avatar_image;
    if (!empty($user_url))
      $html .= '</a>';
    return $html;
  }

  // --------------------------------------------------------------------------
  // Hook functions
  // --------------------------------------------------------------------------

  function comvatar_comment_image_custom($s)
  {
    $html = $this->get_smart_avatar_image_link($user_name, $user_email,
      $user_url, $icon_size, $default_avatar_url). ' '. $s;
    return $html;
  }

  function comvatar_comment_image_default($s)
  {
    $icon_size = COMVATAR_DEFAULT_ICON_SIZE;
    $html = $this->get_smart_avatar_image_link($user_name, $user_email,
      $user_url, $icon_size, $default_avatar_url). ' '. $s;
    return $html;
  }

  function comvatar_xhtml_tag($content='')
  {
    // code borrowed in parts from Skippys Gravatar code
    $matches = array();
    $replacement = array();
    $counter = 0;

    // look for all instances of <!--comvatar ...--> in the content
    preg_match_all("/<!--[\s]*comvatar[\s]{0,1}(.*?)[\s]*-->/is", $content, $matches);
    foreach ($matches['1'] as $match)
    {
      $c = $this->get_string_type($match, $user_name, $user_email, $user_url);
      $replacement[$counter] = $this->get_smart_avatar_image_link($user_name, $user_email,
        $user_url, $icon_size, $default_avatar_url);
      $counter++;
    }
    for ($n = 0; $n <= $counter; $n++)
      $content = str_replace($matches[0][$n], $replacement[$n], $content);
    return $content;
  }

  function comvatar_comment_discover_favicon($comment_ID)
  {
    global $wpdb;

    if ($this->m_favatar_plugin_installed)
    {
      if (COMVATAR_FAVATAR_REDIRECTION)
      {
        $url = $wpdb->get_var("SELECT comment_author_url FROM `$wpdb->comments` WHERE `comment_ID` = '$comment_ID'");
        if (!isset($url) || empty($url) || !$this->m_snoopy->fetch($url))
          return $comment_ID;
        $redirected_url = $this->m_snoopy->lastredirectaddr;
        if (!isset($redirected_url) || empty($redirected_url))
          return $comment_ID;
        $wpdb->query("UPDATE `$wpdb->comments` SET `comment_author_url` = '$redirected_url' WHERE `comment_ID` = '$comment_ID' LIMIT 1");
        $comment_ID = discover_favicon($comment_ID);
        $wpdb->query("UPDATE `$wpdb->comments` SET `comment_author_url` = '$url' WHERE `comment_ID` = '$comment_ID' LIMIT 1");
      }
      else
        $comment_ID = discover_favicon($comment_ID);
    }
    return $comment_ID;
  }
}

// --------------------------------------------------------------------------
// template tags
// --------------------------------------------------------------------------

function comvatar_url($name='', $default_avatar_url='', $icon_size='',
  $display=true)
{
  /* Returns an URL to an avatar image in all cases, either to a default one,
  favatar or gravatar.

  name: (string) If empty, it will try to take the user information from the
  current comment. If no comment is processed at this time, the user name
  will be set to the one defined in COMVATAR_ANONYMOUS_NAME, the user email
  and user URL will be left blank. If name is set and is validated as URL,
  the address will be taken to receive the favatar from this URL. If name
  is set and is validated as email address, the address will be taken to
  receive the gravatar from the Gravatar service.

  default_avatar_url: (string) If set and no user name or user email is
  found valid or the script runs into an error this avatar will be displayed.
  If it is empty, the plugin chooses the COMVATAR_ANONYMOUS_AVATAR image from
  the plugin folder COMVATAR_BASE_PATH.

  icon_size: (integer) If left empty, the plugin takes the size from its
  own configuration. If the Gravatars plugin is activated its size will be
  used to display the comvatars. If Gravatars is not activated the default
  icon size COMVATAR_DEFAULT_ICON_SIZE will be used. If icon size is set it
  will override any of the Comvatars default or Gravatars settings.

  display: (boolean) If true, the resulting output will be printed at the
  position in the template where the tag is used and no value will be
  returned. If display is false, no output will be printed but the resulting
  string will be given back to the caller.
  */
  global $g_comvatar;

  $g_comvatar->get_string_type($name, $user_name, $user_email, $user_url);
  $avatar_url = $g_comvatar->get_smart_avatar_url($user_name, $user_email,
    $user_url, $icon_size, $default_avatar_url);
  if (!display)
    return $avatar_url;
  echo $avatar_url;
}

function comvatar_image($name='', $default_avatar_url='', $icon_size='',
  $before='<img class="comvatar"', $after=' />', $display=true)
{
  /* Returns an HTML <img> tag of an avatar image in all cases, either
  of a default one, favatar or gravatar. XHTML compliant.

  The function parameter for the comvatar_image template tag are the
  same as the parameter for comvatar_url template tag. See the above
  function comvatar_url for an explanation of the parameter.
  Additionally there are the following parameter.

  before: (string) Text to place before the image. Defaults to
  <img class=comvatar.

  after: (string) Text to place after the image. Defaults to  />.
  */
  global $g_comvatar;

  $g_comvatar->get_string_type($name, $user_name, $user_email, $user_url);
  $html = $g_comvatar->get_smart_avatar_image($user_name, $user_email,
    $user_url, $icon_size, $default_avatar_url, $before, $after);
  if (!display)
    return $html;
  echo $html;
}

// --------------------------------------------------------------------------
// Plugin initialization
// --------------------------------------------------------------------------

function comvatar_init()
{
  global $g_comvatar;
  $g_comvatar = new Comvatar();
}

if (function_exists('get_currentuserinfo'))
{
  // WP 1.5, get_currentuserinfo already loaded
  comvatar_init();
}
else
{
  // Need to wait until pluggable functions have been loaded
  // e.g. the Gravatars plugin
  add_action('init', 'comvatar_init');
}
?>